#include "testlib.h"
#include <vector>

using namespace std;

constexpr int MAX_N[6] = {
    100 * 1000,
    8,
    100 * 1000,
    100 * 1000,
    100 * 1000,
    100 * 1000,
};

vector<int> read_number(const string& name, int n) {
    int sz = 3 * n - 1;
    string st = inf.readToken("[0,1,\\|]{" + to_string(sz) + "}", name) + "|";
    
    vector<int> result(n);
    for (int i = 0; i < n; ++i) {
        ensuref(
            '0' <= st[3 * i] && st[3 * i] <= '1',
            "wrong format of %s, '0' or '1' expected, but %c found", 
            name.c_str(),
            st[3 * i]
        );
        ensuref(
            '0' <= st[3 * i + 1] && st[3 * i + 1] <= '1',
            "wrong format of %s, '0' or '1' expected, but %c found", 
            name.c_str(),
            st[3 * i + 1]
        );
        ensuref(
            st[3 * i + 2] == '|',
            "wrong format of %s, '|' expected, but %c found", 
            name.c_str(),
            st[3 * i + 2]
        );
        result[i] = ((st[3 * i] - '0') << 1) + (st[3 * i + 1] - '0');
    }
    return result;
}

void valide_number_with_allowed_symb(const vector<int>& num, const string& name, const set<int> allowed_symb) {
    for (int c : num) {
        ensuref(allowed_symb.find(c) != allowed_symb.end(), "%d not allowed in %s", c, name.c_str());
    }
}

int main(int argc, char* argv[]) {
    registerValidation(argc, argv);
    int gr = validator.group().empty() ? 0 : atoi(validator.group().c_str());
    
    int n = inf.readInt(1, MAX_N[n], "n");
    inf.readEoln();
    
    auto a = read_number("a", n);
    inf.readEoln();
    auto b = read_number("b", n);
    inf.readEoln();
    
    if (gr == 2) {
        valide_number_with_allowed_symb(a, "a", {0});
    } else if (gr == 3) {
        valide_number_with_allowed_symb(a, "a", {0, 1});
        valide_number_with_allowed_symb(b, "b", {0, 1});
    } else if (gr == 4) {
        valide_number_with_allowed_symb(a, "a", {0, 1});
        valide_number_with_allowed_symb(b, "b", {0, 2});
    }

    inf.readEof();
}
