#include "testlib.h"
#include <cassert>
#include <map>
#include <numeric>
#include <set>
#include <string>
#include <unordered_map>
#include <unordered_set>
#define ld long double

using namespace std;


const int RND = 1e9;

ld gen_double() {
	return (ld)rnd.next(0, RND) / (ld)RND;
}


map<string, string> params;
set<string> options;


void parseParams(int argc, char* argv[]) {
	for (int i = 1; i < argc; ++i) {
		string param(argv[i]);
		auto pos = param.find('=');
		if (pos == string::npos) {
			options.insert(param);
		} else {
			params[param.substr(0, pos)] = param.substr(pos + 1);
		}
	}
}


void generatePermutation(int* a, int n) {
	iota(a, a + n, 0);
	shuffle(a, a + n);
}



void generateRandomly(int* a, int n, int B) {
	for (int i = 0; i < n; ++i) {
		a[i] = rnd.next(B);
	}
}


void generateSkewedPermutation(int *a, int l, int r, int sk, int& cur) {
	int max_pos = l + rnd.wnext(r - l + 1, sk);
	a[max_pos] = cur--;
	if (l < max_pos) {
		generateSkewedPermutation(a, l, max_pos - 1, sk, cur);
	}
	if (r > max_pos) {
		generateSkewedPermutation(a, max_pos + 1, r, sk, cur);
	}
}

void generateSkewedPermutation(int* a, int n, int sk) {
	int cur = n - 1;
	generateSkewedPermutation(a, 0, n - 1, sk, cur);
}


void generateLadder(int* a, int n, int stepL, int stepR) {
	vector<int> res(n, -1);
	int l = 0, r = n - 1, cur = n;
	while (true) {
		int side = stepR == 0 ? 0 : stepL == 0 ? 1 : rnd.next(2);
		if (side == 0) {
			if (r - l + 1 < stepL) {
				break;
			}
			l += rnd.next(stepL) + 1;
			res[l - 1] = --cur;
		} else {
			if (r - l + 1 < stepR) {
				break;
			}
			r -= rnd.next(stepR) + 1;
			res[r + 1] = --cur;
		}
	}
	vector<int> other_order(cur);
	iota(other_order.begin(), other_order.end(), 0);
	shuffle(other_order.begin(), other_order.end());
	int other_order_ptr = 0;
	for (int i = 0; i < n; ++i) {
		if (res[i] == -1) {
			res[i] = other_order[other_order_ptr++];
		}
	}
	copy(res.begin(), res.end(), a);
}


void monotonous_permutation(int* arr, int n, bool rev) {
	iota(arr, arr + n, 0);
	if (rev) {
		reverse(arr, arr + n);
	}
}

void monotonous(int* arr, int n, int B, bool rev) {
	for (int i = 0; i < n; ++i) {
		arr[i] = rnd.next(B);
	}
	sort(arr, arr + n);
	if (rev) {
		reverse(arr, arr + n);
	}
}


void chances(int* arr, int n, int B) {
	vector<ld> ch(B);
	for (int i = 0; i < B; ++i) {
		ch[i] = stold(params["chance" + to_string(i + 1)]);
	}
	for (int i = 0; i < n; ++i) {
		ld val = gen_double();
		for (int j = 0; j < B; ++j) {
			if (val < ch[j]) {
				arr[i] = j;
				break;
			} else {
				val -= ch[j];
			}
		}
	}
}


void multitest_123(int* arr, int n, int BLOCK) {
	assert(n % BLOCK == 0);
	for (int i = 0; i < n; i += BLOCK) {
		ld chance1 = gen_double();
		for (int j = 0; j < BLOCK - 1; ++j) {
			arr[i + j] = gen_double() > chance1;
		}
		arr[i + BLOCK - 1] = 2;
	}
}


void pm1(int* arr, int n) {
	for (int i = 1; i < n; ++i) {
		arr[i] = arr[i - 1] + (rnd.next(2) ? 1 : -1);
	}
}


void largeMiddleAns(int* arr, int n) {
	int l = n / 3;
	int r = 2 * n / 3;
	for (int i = l; i <= r; ++i) {
		if (!rnd.next(2)) {
			arr[i] = 1;
		}
	}
}


struct Query {
	int u, v, l, r;
};


void addAllQueries(vector<Query>& queries, int n, int offset = 0) {
	for (int u = 0; u < n; ++u) {
		for (int v = 0; v < n; ++v) {
			for (int l = 0; l < n; ++l) {
				for (int r = l; r < n; ++r) {
					queries.push_back({u + offset, v + offset, l + offset, r + offset});
				}
			}
		}
	}
}


void addSimpleQueries(vector<Query>& queries, int n, int q, int offset = 0) {
	if (q >= n) {
		for (int i = 0; i < n; ++i) {
			queries.push_back({offset + i, offset + i, offset, offset + n - 1});
		}
		q -= n;
	}
	while (q--) {
		int v = rnd.next(n);
		queries.push_back({offset + v, offset + v, offset, offset + n - 1});
	}
}


void addFullSegQueries(vector<Query>& queries, int n, int offset = 0) {
	for (int u = 0; u < n; ++u) {
		for (int v = 0; v < n; ++v) {
			queries.push_back({offset + u, offset + v, offset, offset + n - 1});
		}
	}
}


void addReachQueries(vector<Query>& queries, int n, int offset = 0) {
	for (int u = 0; u < n; ++u) {
		for (int v = 0; v < n; ++v) {
			queries.push_back({offset + u, offset + u, offset + v, offset + v});
		}
	}
}


void addUVRandomQueries(vector<Query>& queries, int n, int q, int offset = 0) {
	while (q--) {
		int v = rnd.next(n), l = rnd.next(n), r = rnd.next(n);
		if (l > r) {
			swap(l, r);
		}
		queries.push_back({v + offset, v + offset, l + offset, r + offset});
	}
}


void addLRQueries(vector<Query>& queries, int n, int q, int offset = 0) {
	while (q--) {
		int u = rnd.next(n), v = rnd.next(n);
		int l, r;
		if (rnd.next(2)) {
			l = rnd.next(max(u, v), n - 1);
			r = rnd.next(max(u, v), n - 1);
		} else {
			l = rnd.next(0, min(u, v));
			r = rnd.next(0, min(u, v));
		}
		if (l > r) {
			swap(l, r);
		}
		queries.push_back({u + offset, v + offset, l + offset, r + offset});
	}
}


void subtask9Queries(vector<Query>& queries, int n, int q, int N, int offset = 0) {
	while (q--) {
		int u = rnd.next(n), v = rnd.next(n);
		queries.push_back({u + offset, v + offset, 0, N - 1});
	}
}



void subtask9Queries_blocks(vector<Query>& queries, int n, int q, int BLOCK) {
	while (q--) {
		int block_id = rnd.next(n / BLOCK);
		int u = block_id * BLOCK + rnd.next(BLOCK), v = block_id * BLOCK + rnd.next(BLOCK);
		queries.push_back({u, v, 0, n - 1});
	}
}


void addRandomQueries(vector<Query>& queries, int n, int q, int offset = 0) {
	while (q--) {
		int u = rnd.next(n), v = rnd.next(n), l = rnd.next(n), r = rnd.next(n);
		if (l > r) {
			swap(l, r);
		}
		queries.push_back({u + offset, v + offset, l + offset, r + offset});
	}
}


void quadraticQ(vector<Query>& queries, int n, int q, int offset = 0) {
	assert(q >= 2 * n * n);
	addFullSegQueries(queries, n, offset);
	addReachQueries(queries, n, offset);
	q -= 2 * n * n;
	while (q--) {
		int u = rnd.next(n), v = rnd.next(n), l = rnd.next(n), r = rnd.next(n);
		if (l > r) {
			swap(l, r);
		}
		queries.push_back({u + offset, v + offset, l + offset, r + offset});
	}
}


const int MAX_N = 5e5 + 10;


namespace solution {
	int n, arr[MAX_N], par[2][MAX_N], greaterR[MAX_N];

	void calcLeftTree(int n, int* arr, int* L) {
		vector<int> st;
		for (int i = n - 1; i >= 0; --i) {
			while (!st.empty() && arr[i] >= arr[st.back()]) {
				st.pop_back();
			}
			greaterR[i] = st.empty() ? n : st.back();
			st.push_back(i);
		}
		vector<vector<int>> to_remove(n);
		set<int> edgeL;
		for (int i = 0; i < n; ++i) {
			for (auto rem : to_remove[i]) {
				edgeL.erase(rem);
			}
			L[i] = edgeL.empty() ? -1 : *prev(edgeL.end());
			edgeL.insert(i);
			if (greaterR[i] < n) {
				to_remove[(i + greaterR[i] + 1) / 2].push_back(i);
			}
		}
	}

	void load(int *_arr, int _n) {
		n = _n;
		copy(_arr, _arr + n, arr);
		calcLeftTree(n, arr, par[0]);
		reverse(arr, arr + n);
		calcLeftTree(n, arr, par[1]);
		reverse(par[1], par[1] + n);
		reverse(arr, arr + n);
		for (int i = 0; i < n; ++i) {
			par[1][i] = n - 1 - par[1][i];
		}
	}
};


void addBalancedQueries(vector<Query>& queries, int n, int* arr, int q, int offset = 0) {
	solution::load(arr, n);
	int d = 0;
	int _n = n;
	while (_n) {
		d++;
		_n >>= 1;
	}
	assert(q >= n);
	if (!options.count("ignoreSimple")) {
		for (int i = 0; i < n; ++i) {
			queries.push_back({i + offset, i + offset, offset, n - 1 + offset});
		}
		q -= n;
	}
	while (q--) {
		int u, v;
		int vert_type = rnd.next(3);
		if (vert_type == 0) {
			u = rnd.next(n);
			v = rnd.next(n);
		} else if (vert_type == 1) {
			int _d = rnd.next(0, d);
			u = rnd.next(0, n - 1 - _d);
			v = u + _d;
		} else {
			while (true) {
				u = rnd.next(n);
				int dir = rnd.next(2);
				if (solution::par[dir][u] >= 0 && solution::par[dir][u] < n) {
					v = solution::par[dir][u];
					break;
				}
			}
		}
		int l, r;
		if (rnd.next(2) == 0) {
			l = 0;
			r = n - 1;
		} else {
			l = rnd.next(n);
			r = rnd.next(n);
			if (l > r) {
				swap(l, r);
			}
		}
		queries.push_back({u + offset, v + offset, l + offset, r + offset});
	}
}


void addNoise(int* arr, int n, int noise) {
	for (int it = 0; it < noise; ++it) {
		int pos = rnd.next(0, n - 2);
		swap(arr[pos], arr[pos + 1]);
	}
}


void askAbout(vector<Query>& queries, int n, vector<int> pos) {
	for (auto i : pos) {
		queries.push_back({i, i, 0, n - 1});
	}
	params["q"] = to_string(stoi(params["q"]) - pos.size());
}


int n, C, arr[MAX_N];
vector<Query> queries;


void distributeValues() {
	vector<int> coords;
	for (int i = 0; i < n; ++i) {
		coords.push_back(arr[i]);
	}
	sort(coords.begin(), coords.end());
	coords.erase(unique(coords.begin(), coords.end()), coords.end());
	assert(coords.size() <= C);
	set<int> used;
	for (auto x : coords) {
		while (true) {
			int y = rnd.next(1, C);
			if (!used.count(y)) {
				used.insert(y);
				break;
			}
		}
	}
	vector<int> _used(used.begin(), used.end());
	unordered_map<int, int> mapping;
	for (int i = 0; i < coords.size(); ++i) {
		mapping[coords[i]] = _used[i];
	}
	for (int i = 0; i < n; ++i) {
		arr[i] = mapping[arr[i]];
	}
}


void make_distinct() {
	vector<pair<int, int>> order;
	for (int i = 0; i < n; ++i) {
		order.emplace_back(arr[i], i);
	}
	sort(order.begin(), order.end());
	for (int i = 0; i < n; ++i) {
		arr[i] = lower_bound(order.begin(), order.end(), make_pair(arr[i], i)) - order.begin();
	}
}


void print() {
	if (options.count("distinct")) {
		make_distinct();
	}
	if (!options.count("dont_distribute")) {
		distributeValues();
	}
	if (!options.count("dont_shuffle")) {
		shuffle(queries.begin(), queries.end());
	}
	cout << n << '\n';
	for (int i = 0; i < n; ++i) {
		if (i) {
			cout << ' ';
		}
		cout << arr[i];
	}
	cout << '\n';
	cout << queries.size() << '\n';
	for (auto query : queries) {
		cout << query.u + 1 << ' ' << query.v + 1 << ' ' << query.l + 1 << ' ' << query.r + 1 << '\n';
	}
}


void anti_slow_binup() {
	generateLadder(arr, n, 2, 2);
	int l = n / 3;
	int r = 2 * n / 3;
	fill(arr + l, arr + r + 1, -1);
	int q = stoi(params["q"]);
	while (q--) {
		int u = rnd.next(l, r);
		int L = rnd.next(n), R = rnd.next(n);
		if (L > R) {
			swap(L, R);
		}
		queries.push_back({u, u, L, R});
	}
}


void anti_slow_lca() {
	int l = n / 3;
	int r = 2 * n / 3;
	generateLadder(arr, l, 2, 0);
	generateLadder(arr + l, r - l + 1, 2, 0);
	generateLadder(arr + r, n - r, 2, 0);
	for (int i = l; i < n; ++i) {
		arr[i] -= n / 3;
	}
	int q = stoi(params["q"]);
	while (q--) {
		int u = rnd.next(l, r);
		int v = rnd.next(r + 1, n - 1);
		int l, r;
		if (options.count("lr")) {
			l = 0;
			r = n - 1;
		} else {
			l = rnd.next(u);
			r = rnd.next(u);
			if (rnd.next(2)) {
				swap(u, v);
			}
			if (l > r) {
				swap(l, r);
			}
		}
		queries.push_back({u, v, l, r});
	}
}


void reverse_test() {
	reverse(arr, arr + n);
	for (auto& query: queries) {
		query.u = n - 1 - query.u;
		query.v = n - 1 - query.v;
		query.l = n - 1 - query.l;
		query.r = n - 1 - query.r;
		swap(query.l, query.r);
	}
}


void generate() {
	n = stoi(params["n"]);
	if (params["test"] == "anti_slow_binup") {
		anti_slow_binup();
		return;
	}
	if (params["test"] == "anti_slow_lca") {
		anti_slow_lca();
		if (params.count("noise")) {
			addNoise(arr, n, stoi(params["noise"]));
		}
		if (options.count("reverse")) {
			reverse_test();
		}
		return;
	}

	if (params["array"] == "permutation") {
		generatePermutation(arr, n);
	} else if (params["array"] == "random") {
		generateRandomly(arr, n, stoi(params["B"]));
	} else if (params["array"] == "skewed") {
		generateSkewedPermutation(arr, n, stoi(params["sk"]));
	} else if (params["array"] == "ladder") {
		int stepL = params.count("stepL") ? stoi(params["stepL"]) : 0;
		int stepR = params.count("stepR") ? stoi(params["stepR"]) : 0;
		generateLadder(arr, n, stepL, stepR);
	} else if (params["array"] == "increasing" || params["array"] == "decreasing") {
		if (params.count("B")) {
			monotonous(arr, n, stoi(params["B"]), params["array"] == "decreasing");
		} else {
			monotonous_permutation(arr, n, params["array"] == "decreasing");
		}
	} else if (params["array"] == "constant") {
		fill(arr, arr + n, stoi(params["B"]));
		options.insert("dont_distribute");
	} else if (params["array"] == "12_21") {
		arr[1] = 1;
		arr[n - 2] = 1;
		generateRandomly(arr + 2, n - 4, 2);
		askAbout(queries, n, {0, 1, n - 1, n - 2});
	} else if (params["array"] == "2_2111") {
		arr[0] = 1;
		arr[n - 4] = 1;
		generateRandomly(arr + 1, n - 5, 2);
		askAbout(queries, n, {0, n - 4, n - 3, n - 2, n - 1});
	} else if (params["array"] == "chances") {
		chances(arr, n, C);
	} else if (params["array"] == "multitest_123") {
		multitest_123(arr, n, 100);
	} else if (params["array"] == "cnt2") {
		vector<int> all_pos(n);
		iota(all_pos.begin(), all_pos.end(), 0);
		shuffle(all_pos.begin(), all_pos.end());
		for (int i = 0; i < stoi(params["cnt2"]); ++i) {
			arr[all_pos[i]] = 1;
		}
	} else if (params["array"] == "pm1") {
		pm1(arr, n);
	} else if (params["array"] == "large_middle_ans") {
		largeMiddleAns(arr, n);
	} else {
		assert(false);
	}

	if (params.count("noise")) {
		addNoise(arr, n, stoi(params["noise"]));
	}
	if (params.count("flipped")) {
		for (int i = 0; i < n; ++i) {
			arr[i] = -arr[i];
		}
	}

	if (params["queries"] == "all") {
		addAllQueries(queries, n);
	} else if (params["queries"] == "simple") {
		addSimpleQueries(queries, n, stoi(params["q"]));
	} else if (params["queries"] == "quadratic") {
		addFullSegQueries(queries, n);
		addReachQueries(queries, n);
	} else if (params["queries"] == "balanced") {
		addBalancedQueries(queries, n, arr, stoi(params["q"]));
	} else if (params["queries"] == "uv_random") {
		addUVRandomQueries(queries, n, stoi(params["q"]));
	} else if (params["queries"] == "lr_random") {
		addLRQueries(queries, n, stoi(params["q"]));
	} else if (params["queries"] == "subtask9") {
		subtask9Queries(queries, n, stoi(params["q"]), n);
	} else if (params["queries"] == "subtask9_block100") {
		subtask9Queries_blocks(queries, n, stoi(params["q"]), 100);
	} else if (params["queries"] == "random") {
		addRandomQueries(queries, n, stoi(params["q"]));
	}
}


void generate_multitest() {
	int max_n = stoi(params["max_n"]);
	int test_sz = stoi(params["test_sz"]);
	int max_q = stoi(params["max_q"]);
	int T = max_n / (3 * test_sz + 1);
	int q_test = max_q / T;
	n = T * (3 * test_sz + 1);
	fill(arr, arr + n, -1);
	vector<int> possibleB;
	for (int B = test_sz; B >= 2; B = (int)floor((ld)B * 0.9)) {
		possibleB.push_back(B);
	}
	for (int pos = 0; pos < n; pos += (3 * test_sz + 1)) {
		if (params["array"] == "permutation") {
			generatePermutation(arr + pos + test_sz, test_sz);
		} else if (params["array"] == "random") {
			generateRandomly(arr + pos + test_sz, test_sz, rnd.any(possibleB));
		} else if (params["array"] == "skewed") {
			int ampl = stoi(params["ampl"]);
			generateSkewedPermutation(arr + pos + test_sz, test_sz, rnd.next(-ampl, ampl));
		}
		arr[pos + 3 * test_sz] = test_sz;
		if (params["queries"] == "quadraticQ") {
			quadraticQ(queries, test_sz, q_test, pos + test_sz);
		} else if (params["queries"] == "reach") {
			addReachQueries(queries, test_sz, pos + test_sz);
		} else if (params["queries"] == "uv_random") {
			addUVRandomQueries(queries, test_sz, q_test, pos + test_sz);
		} else if (params["queries"] == "lr_random") {
			addLRQueries(queries, test_sz, q_test, pos + test_sz);
		} else if (params["queries"] == "subtask9") {
			subtask9Queries(queries, test_sz, q_test, n, pos + test_sz);
		} else if (params["queries"] == "random") {
			addRandomQueries(queries, test_sz, q_test, pos + test_sz);
		}
	}
	if (params["queries"] == "simple") {
		addSimpleQueries(queries, n, max_q);
	}
	assert(queries.size() <= max_q);
}


int main(int argc, char* argv[]) {
	registerGen(argc, argv, 1);
	parseParams(argc, argv);
	C = stoi(params["C"]);
	if (options.count("multitest")) {
		generate_multitest();
	} else {
		generate();
	}
	print();
}
