#include "testlib.h"
#include <algorithm>
#include <vector>
#include <string>

using namespace std;

enum class QueriesType { any, basic, LR, u_eq_v, full_segment };

struct Constrains {
	int maxN, maxC, maxQ;
	QueriesType queries;
	bool checkDistinct;
};

const int MAX_N = 5e5;
const int MAX_C = 1e9;
const int MAX_Q = 5e5;

vector<Constrains> groups = {
	{MAX_N, MAX_C, MAX_Q, QueriesType::any, false},
	{100, MAX_C, 1000, QueriesType::any, false},
	{2000, MAX_C, MAX_Q, QueriesType::any, true},
	{(int)2e5, 2, (int)1e5, QueriesType::basic, false},
	{(int)2e5, 3, (int)1e5, QueriesType::basic, false},
	{10000, MAX_C, (int)10000, QueriesType::basic, false},
	{MAX_N, MAX_C, MAX_Q, QueriesType::basic, false},
	{MAX_N, MAX_C, MAX_Q, QueriesType::u_eq_v, false},
	{MAX_N, MAX_C, MAX_Q, QueriesType::LR, false},
	{MAX_N, MAX_C, MAX_Q, QueriesType::full_segment, false},
	{MAX_N, MAX_C, MAX_Q, QueriesType::any, false}
};

const char* incorrectQueryMessage = "The query does not satisfy the subtask";

int main(int argc, char* argv[]) {
	registerValidation(argc, argv);
	auto constrains = groups[validator.group() == "" ? 0 : stoi(validator.group())];
	int n = inf.readInt(1, constrains.maxN, "n");
	inf.readEoln();
	vector<int> arr = inf.readInts(n, 1, constrains.maxC, "a");
	inf.readEoln();
	int q = inf.readInt(1, constrains.maxQ, "q");
	inf.readEoln();
	for (int i = 0; i < q; ++i) {
		int u = inf.readInt(1, n, "u");
		inf.readSpace();
		int v = inf.readInt(1, n, "v");
		inf.readSpace();
		int l = inf.readInt(1, n, "l");
		inf.readSpace();
		int r = inf.readInt(1, n, "r");
		inf.readEoln();
		ensuref(l <= r, "The bounds of a segment are invalid");
		switch (constrains.queries) {
			case QueriesType::basic:
				ensuref(u == v && l == 1 && r == n, "%s", incorrectQueryMessage);
				break;
			case QueriesType::LR:
				ensuref(r <= min(u, v) || l >= max(u, v), "%s", incorrectQueryMessage);
				break;
			case QueriesType::u_eq_v:
				ensuref(u == v, "%s", incorrectQueryMessage);
				break;
			case QueriesType::full_segment:
				ensuref(l == 1 && r == n, "%s", incorrectQueryMessage);
				break;
			case QueriesType::any:
				break;
		}
	}
	if (constrains.checkDistinct) {
		sort(arr.begin(), arr.end());
		for (int i = 1; i < (int)arr.size(); ++i) {
			ensuref(arr[i - 1] < arr[i], "The heights should be distinct");
		}
	}
	inf.readEof();
}
