#include <algorithm>
#include <iostream>
#include <set>
#include <vector>

using namespace std;

struct SegmentTree {
	vector<int> t;
	int n;

	SegmentTree() {}
	SegmentTree(int _n) {
		n = 1;
		while (n < _n) {
			n <<= 1;
		}
		t.resize(2 * n);
	}

	void augment_segment(int l, int r, int val) {
		l += n;
		r += n;
		while (l <= r) {
			if (l & 1) {
				t[l] += val;
				l++;
			}
			if (!(r & 1)) {
				t[r] += val;
				r--;
			}
			l >>= 1;
			r >>= 1;
		}
	}

	int get(int pos) {
		int res = 0;
		pos += n;
		while (pos) {
			res += t[pos];
			pos >>= 1;
		}
		return res;
	}
};

const int MAX_N = 1e6 + 42;
int greaterR[MAX_N];
SegmentTree T;

void calcLeftTree(int n, int* arr, int* L) {
	vector<int> st;
	for (int i = n - 1; i >= 0; --i) {
		while (!st.empty() && arr[i] >= arr[st.back()]) {
			st.pop_back();
		}
		greaterR[i] = st.empty() ? n : st.back();
		st.push_back(i);
	}
	vector<vector<int>> to_remove(n);
	set<int> edgeL;
	for (int i = 0; i < n; ++i) {
		for (auto rem : to_remove[i]) {
			edgeL.erase(rem);
		}
		L[i] = edgeL.empty() ? -1 : *prev(edgeL.end());
		edgeL.insert(i);
		if (greaterR[i] < n) {
			to_remove[(i + greaterR[i] + 1) / 2].push_back(i);
		}
	}
}

const int LG = 20;
int n, a[MAX_N], par[2][MAX_N], up[2][MAX_N][LG];

void calc_up() {
	for (int i = 0; i < n; ++i) {
		up[0][i][0] = par[0][i] == -1 ? i : par[0][i];
		up[1][i][0] = par[1][i] == n ? i : par[1][i];
	}
	for (int lvl = 1; lvl < LG; ++lvl) {
		for (int i = 0; i < n; ++i) {
			up[0][i][lvl] = up[0][up[0][i][lvl - 1]][lvl - 1];
		}
		for (int i = n - 1; i >= 0; --i) {
			up[1][i][lvl] = up[1][up[1][i][lvl - 1]][lvl - 1];
		}
	}
}

vector<vector<int>> ch[2];
int tin[2][MAX_N], tout[2][MAX_N], depth[2][MAX_N], timer;

int get_depth(int tree_id, int v) {
	return v >= 0 && v < n ? depth[tree_id][v] : -1;
}

void dfs(int tree_id, int v) {
	tin[tree_id][v] = timer++;
	for (auto v1 : ch[tree_id][v]) {
		depth[tree_id][v1] = depth[tree_id][v] + 1;
		dfs(tree_id, v1);
	}
	tout[tree_id][v] = timer - 1;
}

void process_tree(int tree_id) {
	ch[tree_id].resize(n);
	for (int i = 0; i < n; ++i) {
		if (par[tree_id][i] >= 0 && par[tree_id][i] < n) {
			ch[tree_id][par[tree_id][i]].push_back(i);
		}
	}
	timer = 0;
	for (int i = 0; i < n; ++i) {
		if (par[tree_id][i] < 0 || par[tree_id][i] >= n) {
			dfs(tree_id, i);
		}
	}
}

int upper(int tree_id, int u, int v) {
	return tin[tree_id][u] <= tin[tree_id][v] && tout[tree_id][v] <= tout[tree_id][u];
}

int lca(int tree_id, int u, int v) {
	if (upper(tree_id, u, v)) {
		return u;
	}
	for (int i = LG - 1; i >= 0; --i) {
		if (!upper(tree_id, up[tree_id][u][i], v)) {
			u = up[tree_id][u][i];
		}
	}
	return par[tree_id][u];
}

int ascend(int tree_id, int v, int bound) {
	if (tree_id ? v >= bound : v <= bound) {
		return v;
	}
	for (int i = LG - 1; i >= 0; --i) {
		if (!(tree_id ? up[tree_id][v][i] >= bound : up[tree_id][v][i] <= bound)) {
			v = up[tree_id][v][i];
		}
	}
	return par[tree_id][v];
}

struct Query {
	int u, v, l, r;
};

const int MAX_Q = 1e6 + 42;
vector<vector<int>> q_by_v;
int ans[MAX_Q];
Query queries[MAX_Q];

void process_query(int id) {
	auto [u, v, l, r] = queries[id];
	if (l < u) {
		int w = lca(0, u, v), w1 = ascend(0, w, min(u - 1, r)), w2 = ascend(0, w, l - 1);
		ans[id] += get_depth(0, w1) - get_depth(0, w2);
	}
	if (r > v) {
		int w = lca(1, u, v), w1 = ascend(1, w, max(v + 1, l)), w2 = ascend(1, w, r + 1);
		ans[id] += get_depth(1, w1) - get_depth(1, w2);
	}
	int l1 = max(u, l), r1 = min(r, v);
	if (l1 <= r1) {
		q_by_v[v].push_back(id);
	}
}

void dfs(int v) {
	T.augment_segment(tin[1][v], tout[1][v], 1);
	for (auto id : q_by_v[v]) {
		auto [u, v, l, r] = queries[id];
		int u1 = ascend(1, u, max(l, u)), u2 = ascend(1, u, min(r, v) + 1);
		ans[id] += (u1 == n ? 0 : T.get(tin[1][u1])) - (u2 == n ? 0 : T.get(tin[1][u2]));
	}
	for (auto v1 : ch[0][v]) {
		dfs(v1);
	}
	T.augment_segment(tin[1][v], tout[1][v], -1);
}

void process_middles() {
	for (int i = 0; i < n; ++i) {
		if (par[0][i] == -1) {
			dfs(i);
		}
	}
}

int main() {
	ios_base::sync_with_stdio(0); cin.tie(0);
	cin >> n;
	for (int i = 0; i < n; ++i) {
		cin >> a[i];
	}
	calcLeftTree(n, a, par[0]);
	reverse(a, a + n);
	calcLeftTree(n, a, par[1]);
	reverse(par[1], par[1] + n);
	for (int i = 0; i < n; ++i) {
		par[1][i] = n - 1 - par[1][i];
	}
	calc_up();
	for (int tree_id = 0; tree_id < 2; ++tree_id) {
		process_tree(tree_id);
	}
	T = SegmentTree(n);
	q_by_v.resize(n);
	int q;
	cin >> q;
	for (int i = 0; i < q; ++i) {
		int u, v, l, r;
		cin >> queries[i].u >> queries[i].v >> queries[i].l >> queries[i].r;
		queries[i].u--;
		queries[i].v--;
		queries[i].l--;
		queries[i].r--;
		if (queries[i].u > queries[i].v) {
			swap(queries[i].u, queries[i].v);
		}
		process_query(i);
	}
	process_middles();
	for (int i = 0; i < q; ++i) {
		cout << ans[i] << '\n';
	}
}
