#include <algorithm>
#include <iostream>
#include <set>
#include <vector>

using namespace std;

const int MAX_N = 1e6 + 42;
int greaterR[MAX_N];

void calcLeftTree(int n, int* arr, int* L) {
	vector<int> st;
	for (int i = n - 1; i >= 0; --i) {
		while (!st.empty() && arr[i] >= arr[st.back()]) {
			st.pop_back();
		}
		greaterR[i] = st.empty() ? n : st.back();
		st.push_back(i);
	}
	vector<vector<int>> to_remove(n);
	set<int> edgeL;
	for (int i = 0; i < n; ++i) {
		for (auto rem : to_remove[i]) {
			edgeL.erase(rem);
		}
		L[i] = edgeL.empty() ? -1 : *prev(edgeL.end());
		edgeL.insert(i);
		if (greaterR[i] < n) {
			to_remove[(i + greaterR[i] + 1) / 2].push_back(i);
		}
	}
}

const int LG = 20;
int n, a[MAX_N], par[2][MAX_N], up[2][MAX_N][LG];

void calc_up() {
	for (int i = 0; i < n; ++i) {
		up[0][i][0] = par[0][i] == -1 ? i : par[0][i];
		up[1][i][0] = par[1][i] == n ? i : par[1][i];
	}
	for (int lvl = 1; lvl < LG; ++lvl) {
		for (int i = 0; i < n; ++i) {
			up[0][i][lvl] = up[0][up[0][i][lvl - 1]][lvl - 1];
		}
		for (int i = n - 1; i >= 0; --i) {
			up[1][i][lvl] = up[1][up[1][i][lvl - 1]][lvl - 1];
		}
	}
}

vector<vector<int>> ch[2];
int tin[2][MAX_N], tout[2][MAX_N], depth[2][MAX_N], timer;

int get_depth(int tree_id, int v) {
	return v >= 0 && v < n ? depth[tree_id][v] : -1;
}

void dfs(int tree_id, int v) {
	tin[tree_id][v] = timer++;
	for (auto v1 : ch[tree_id][v]) {
		depth[tree_id][v1] = depth[tree_id][v] + 1;
		dfs(tree_id, v1);
	}
	tout[tree_id][v] = timer - 1;
}

void process_tree(int tree_id) {
	ch[tree_id].resize(n);
	for (int i = 0; i < n; ++i) {
		if (par[tree_id][i] >= 0 && par[tree_id][i] < n) {
			ch[tree_id][par[tree_id][i]].push_back(i);
		}
	}
	timer = 0;
	for (int i = 0; i < n; ++i) {
		if (par[tree_id][i] < 0 || par[tree_id][i] >= n) {
			dfs(tree_id, i);
		}
	}
}

int ascend(int tree_id, int v, int bound) {
	if (tree_id ? v >= bound : v <= bound) {
		return v;
	}
	for (int i = LG - 1; i >= 0; --i) {
		if (!(tree_id ? up[tree_id][v][i] >= bound : up[tree_id][v][i] <= bound)) {
			v = up[tree_id][v][i];
		}
	}
	return par[tree_id][v];
}

int main() {
	ios_base::sync_with_stdio(0); cin.tie(0);
	cin >> n;
	for (int i = 0; i < n; ++i) {
		cin >> a[i];
	}
	calcLeftTree(n, a, par[0]);
	reverse(a, a + n);
	calcLeftTree(n, a, par[1]);
	reverse(par[1], par[1] + n);
	for (int i = 0; i < n; ++i) {
		par[1][i] = n - 1 - par[1][i];
	}
	calc_up();
	for (int tree_id = 0; tree_id < 2; ++tree_id) {
		process_tree(tree_id);
	}
	int q;
	cin >> q;
	for (int i = 0; i < q; ++i) {
		int u, v, l, r;
		cin >> u >> v >> l >> r;
		u--;
		v--;
		l--;
		r--;
		if (u < v) {
			swap(u, v);
		}
		int res_l = get_depth(0, ascend(0, u, r)) - get_depth(0, ascend(0, u, l - 1));
		int res_r = get_depth(1, ascend(1, v, l)) - get_depth(1, ascend(1, v, r + 1));
		cout << res_l + res_r - (int)(l <= u && u <= r) << '\n';
	}
}
